/*	**************************************************************************************
    *									overlay.js
    *									==========
    *
    *	Author:	Geva Zeichner
	*	Forum:	http://forums.mozillazine.org/viewtopic.php?t=408043
    *	Date:	15/04/2006
    *	Edited:	29/04/2006
    **************************************************************************************/

window.addEventListener("load", function(e){IMDb_Preview.handle_load(e);}, false);

var IMDb_Preview =
{
	/*************************************************************************************
	 *	Handle_Load(e)
	 *	==============
	 *	Listeners, var defs, inits...
	 *************************************************************************************/
	handle_load: function(e)
	{
		// Shhh... listen...
		window.addEventListener("mouseover", function(e){IMDb_Preview.handle_mouse_over(e);}, false);
		window.addEventListener("mousemove", function(e){IMDb_Preview.handle_mouse_move(e);}, false);
		
		// Set variables
		this.prefs = Components.classes['@mozilla.org/preferences-service;1'].
						getService(Components.interfaces.nsIPrefService).
						getBranch('extensions.imdb_preview.');
		this.title_regexp = /^((http:\/\/)(www\.)?(us\.)?(uk\.)?((imdb\.com)(.*)(\/title\/tt\d{7}\/)))$/;
		this.move_timer = "";
		this.mouse_over_url_target = "";
		this.loaded_url = " ";
		this.last_owner_doc = "";
		this.http_req = new XMLHttpRequest();
		this.is_showing = false;
		this.page_x = 0;
		this.page_y = 0;
		
		this.init_preview_settings();
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Over(e)
	 *	====================
	 *	Gets info about what we stepped into
	 *************************************************************************************/
	handle_mouse_over: function(e)
	{
		this.page_x = e.pageX;
		this.page_y = e.pageY;
		this.mouse_over_url_target = unescape(e.target.href);
		this.last_owner_doc = e.originalTarget.ownerDocument;
		
		this.terminate_preview();
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Move(e)
	 *	====================
	 *	Checks if we are halting somewhere
	 *************************************************************************************/
	handle_mouse_move: function(e)
	{
		clearTimeout(this.move_timer);
		this.move_timer = setTimeout(function(){IMDb_Preview.handle_mouse_halt();},
									 this.prefs.getIntPref("halt_time"));
	},
	
	/*************************************************************************************
	 *	Handle_Mouse_Halt()
	 *	===================
	 *	Checks if we halted on the right spot
	 *************************************************************************************/
	handle_mouse_halt: function()
	{
		// If standing on url that is different from the one stood on before && url match
		if ((this.mouse_over_url_target != this.loaded_url) &&
			(this.mouse_over_url_target.match(this.title_regexp)))
		{
			// Load url
			this.load_target(this.mouse_over_url_target);
		}
		this.loaded_url = this.mouse_over_url_target;
	},
	
	/*************************************************************************************
	 *	Load_Target(url_target)
	 *	=======================
	 *	Retrives the target's html code and triggers its processing
	 *************************************************************************************/
	load_target: function(url_target)
	{
		this.is_showing = true;
		dump("loading\n");
		
		// Load 'loading...' poster
		this.load_img(this.loading_img_path);
		
		// Load target
		this.http_req.open('GET', url_target, true);
		this.http_req.onreadystatechange = function()
		{
			// If loading completed
			if (IMDb_Preview.http_req.readyState == 4)
			IMDb_Preview.timer = setTimeout( function ()
			{
				try
				{
					// With no error
					if (IMDb_Preview.http_req.status == 200)
						IMDb_Preview.process_html(IMDb_Preview.http_req.responseText);
					else
						dump("Error loading page\n");
				}
				catch(err)
				{
					dump("Page not loaded\n");
				}
			}, 0);
		};
		this.http_req.send(null);
	},
	
	/*************************************************************************************
	 *	Process_HTML(html_source)
	 *	=========================
	 *	Gets the page's html code.
	 *	The poster's img src is extracted from the html and then sent to show_preview_img.
	 *************************************************************************************/
	process_html: function(html_source)
	{
		var title_start_pos = html_source.indexOf('<title>', poster_pos) + 7;
		var title_end_pos = html_source.indexOf('</title>', title_start_pos);
		var title_name = html_source.substring(title_start_pos, title_end_pos);
		
		var poster_pos = html_source.indexOf('<a name="poster"');
		
		if (poster_pos == -1)
			dump("Poster Not Submitted\n\n");
		else
		{
			var img_src_start_pos = html_source.indexOf('src="', poster_pos) + 5;
			var img_src_end_pos = html_source.indexOf('" height', img_src_start_pos);
			var img_src = html_source.substring(img_src_start_pos, img_src_end_pos);
			
			dump(title_name + ":\n" + img_src + "\n\n");
			
			// Load poster
			this.load_img(img_src);
		}
	},
	
	/*************************************************************************************
	 *	Init_Preview_Settings()
	 *	=======================
	 *	
	 *************************************************************************************/
	init_preview_settings: function()
	{
		this.loading_img_path = "chrome://browser/skin/Throbber.gif";
		
		this.poster_img = window._content.document.createElement("img");
		this.poster_img.addEventListener("load", function(e){IMDb_Preview.handle_img_load(e);}, false);
		this.poster_img.src = "";
		
		this.wrap_div = window._content.document.createElement("div");
		this.wrap_div.appendChild(this.poster_img);
		
		// Poster style settings
		this.wrap_div.style.visibility = "hidden";
		this.wrap_div.style.textTop = "true";
		this.wrap_div.style.zIndex = "1000";
		this.wrap_div.style.position = "absolute";
		this.wrap_div.style.left = "0px";
		this.wrap_div.style.top = "0px";
		this.wrap_div.style.backgroundColor = "white";
	},
	
	/*************************************************************************************
	 *	Load_IMG(img_src)
	 *	=================
	 *	Loads an image into the img object
	 *************************************************************************************/
	load_img: function(img_src)
	{
		this.wrap_div.style.visibility = "hidden";
		this.poster_img.src = img_src;
		this.last_owner_doc.body.appendChild(this.wrap_div);
	},
	
	/*************************************************************************************
	 *	Handle_IMG_Load(e)
	 *	==================
	 *	When the img finshed to load
	 *************************************************************************************/
	handle_img_load: function(e)
	{
		if (this.poster_img.src != "")
			this.show_preview();
	},
	
	/*************************************************************************************
	 *	Show_Preview()
	 *	==============
	 *	Positions the preview and reveals it
	 *************************************************************************************/
	show_preview: function()
	{
		var doc = this.last_owner_doc;
		var top, left = 0;
		
		// If in upper half of window
		if (this.page_y < (doc.defaultView.innerHeight / 2) + doc.defaultView.pageYOffset)
			
			// Display preview below cursor
			top = this.page_y + 32;
		else
			top = this.page_y - this.poster_img.height - 10;
		
		// If in the left half of window
		if (this.page_x < (doc.defaultView.innerWidth / 2) + doc.defaultView.pageXOffset)
			
			// Display preview to the right of the cursor
			left = this.page_x;
		else
			left = this.page_x - this.poster_img.width;
		
		this.wrap_div.style.top = top + "px";
		this.wrap_div.style.left = left + "px";
		
		// Reveals the preview
		this.wrap_div.style.visibility = "visible";
	},
	
	/*************************************************************************************
	 *	Terminate_Preview()
	 *	===================
	 *	I'll be back...
	 *************************************************************************************/
	terminate_preview: function()
	{
		if (this.is_showing == true)
		{
			this.http_req.abort();
			this.wrap_div.style.visibility = "hidden";
			this.poster_img.src = "";
			this.loaded_url = " ";
			this.is_showing = false;
		}
	}
};